import ldap
import ldap.filter

from ldap_groups import settings
from django.contrib.auth.models import User

from ldap_groups.models import LDAPGroup


class BaseGroupMembershipBackend(object):
    """
    Base class for implementing an authentication backend which authenticates
    against LDAP and sets Django group membership based on LDAP Organizational
    Unit (OU) membership.
    """
    def authenticate(self, username=None, password=None):
        """
        Attempts to bind the provided username and password to LDAP.

        A successful LDAP bind authenticates the user.
        """
        raise NotImplementedError

    def bind_ldap(self, username, password):
        """
        Implements the specific logic necessary to bind a given username and
        password to the particular LDAP server.

        Override this method for each new variety of LDAP backend.
        """
        raise NotImplementedError

    def get_or_create_user(self, username, password):
        """
        Attempts to get the user from the Django db; failing this, creates a
        django.contrib.auth.models.User from details pulled from the specific
        LDAP backend.

        Override this method for each new variety of LDAP backend.
        """
        raise NotImplementedError

    def get_user(self, user_id):
        """
        Implements the logic to retrieve a specific user from the Django db.
        """
        try:
            return User.objects.get(pk=user_id)
        except User.DoesNotExist:
            return None

    def set_memberships_from_ldap(self, user, membership):
        """
        Assigns user to specific django.contrib.auth.models.Group groups based
        on ldap_group mappings created by the site admin, also assigns staff
        or superuser privileges based on those same mappings.
        """
        ldap_groups = LDAPGroup.objects.filter(org_unit__in=membership)
        for l_grp in ldap_groups:
            for grp in l_grp.groups.all():
                user.groups.add(grp)

        staff_groups = ldap_groups.filter(make_staff=True).count()
        if staff_groups > 0:
            user.is_staff = True

        superuser_groups = ldap_groups.filter(make_superuser=True).count()
        if superuser_groups > 0:
            user.is_superuser = True
        user.save()


class ActiveDirectoryGroupMembershipSSLBackend(BaseGroupMembershipBackend):
    def bind_ldap(self, username, password):
        try:
            ldap.set_option(ldap.OPT_X_TLS_CACERTFILE,settings.CERT_FILE)
        except AttributeError:
            pass
        ldap.set_option(ldap.OPT_REFERRALS,0) # DO NOT TURN THIS OFF OR SEARCH WON'T WORK!
        l = ldap.initialize(settings.LDAP_URL)
        l.set_option(ldap.OPT_PROTOCOL_VERSION, 3)
        binddn = "%s@%s" % (username,settings.LDAP_NT4_DOMAIN)
        l.simple_bind_s(binddn,password)
        return l

    def authenticate(self,username=None,password=None):
        if settings.LDAP_URL in [None, '', 'ldap://:389']:
            # LDAP is not configured so just ignore it
            return
        try:
            if len(password) == 0:
                return None
            l = self.bind_ldap(username, password)
            l.unbind_s()
            return self.get_or_create_user(username,password)

        except ImportError:
            pass
        except ldap.INVALID_CREDENTIALS:
            pass

    def get_or_create_user(self, username, password):
        try:
            user = User.objects.get(username=username)
        except User.DoesNotExist:

            try:
                l = self.bind_ldap(username, password)
                # search
                result = l.search_ext_s(
                    settings.SEARCH_DN,
                    ldap.SCOPE_SUBTREE,
                    "sAMAccountName=%s" % username,
                    settings.SEARCH_FIELDS)[0][1]

                if 'memberOf' in result:
                    membership = result['memberOf']
                else:
                    membership = None

                # get email
                if 'mail' in result:
                    mail = result['mail'][0]
                else:
                    mail = None
                # get surname
                if 'sn' in result:
                    last_name = result['sn'][0]
                else:
                    last_name = None

                # get display name
                if 'givenName' in result:
                    first_name = result['givenName'][0]
                else:
                    first_name = None

                l.unbind_s()

                user = User(username=username,first_name=first_name,last_name=last_name,email=mail)

            except Exception as e:
                return None

            user.is_staff = False
            user.is_superuser = False
            user.set_unusable_password()
            user.save()

            self.set_memberships_from_ldap(user, membership)

        return user

class eDirectoryGroupMembershipSSLBackend(BaseGroupMembershipBackend):
    def bind_ldap(self, username, password):
        try:
            ldap.set_option(ldap.OPT_X_TLS_CACERTFILE,settings.CERT_FILE)
        except AttributeError:
            pass
        l = ldap.initialize(settings.LDAP_URL)
        l.set_option(ldap.OPT_PROTOCOL_VERSION, 3)
        l.simple_bind_s(username, password)
        return l

    def authenticate(self,username=None,password=None):
        try:
            if len(password) == 0:
                return None
            l = self.bind_ldap(settings.BIND_USER, settings.BIND_PASSWORD)
            base = settings.SEARCH_DN
            scope = ldap.SCOPE_SUBTREE
            retrieve_attributes = ['cn']

            filtered_name = ldap.filter.escape_filter_chars(username)
            filter = 'cn=%s' % filtered_name

            results = l.search_s(base, scope, filter, retrieve_attributes)
            candidate_dns = [result[0] for result in results]

            l.unbind()
            for dn in candidate_dns:
                try:
                    l = ldap.initialize(settings.LDAP_URL)
                    l.set_option(ldap.OPT_PROTOCOL_VERSION, 3)
                    l.simple_bind_s(dn, password)
                except ldap.INVALID_CREDENTIALS:
                    l.unbind()
                    continue
                l.unbind()
                return self.get_or_create_user(dn, password)

        except ImportError:
            pass
        except ldap.INVALID_CREDENTIALS:
            pass

    def get_or_create_user(self, username, password):
        stripped_name = ''
        if username.lower().startswith('cn='):
            stripped_name = username.split(',')[0][3:].lower()
        try:
            user = User.objects.get(username=stripped_name)
        except User.DoesNotExist:
            try:
                l = self.bind_ldap(settings.BIND_USER, settings.BIND_PASSWORD)
                # search
                result = l.search_ext_s(settings.SEARCH_DN,
                                        ldap.SCOPE_SUBTREE,
                                        "cn=%s" % stripped_name,
                                        settings.SEARCH_FIELDS)[0][1]
                l.unbind_s()

                if 'groupMembership' in result:
                    membership = result['groupMembership']
                else:
                    membership = None

                # get email
                if 'mail' in result:
                    mail = result['mail'][0]
                else:
                    mail = None
                # get surname
                if 'sn' in result:
                    last_name = result['sn'][0]
                else:
                    last_name = None

                # get display name
                if 'givenName' in result:
                    first_name = result['givenName'][0]
                else:
                    first_name = None


                user = User(username=stripped_name,first_name=first_name,last_name=last_name,email=mail)
            except Exception as e:
                return None

            user.is_staff = False
            user.is_superuser = False
            user.set_unusable_password()
            user.save()

            self.set_memberships_from_ldap(user, membership)

        return user

class OpenDirectoryBackend(BaseGroupMembershipBackend):
    def bind_ldap(self, username, password):
        try:
            ldap.set_option(ldap.OPT_X_TLS_CACERTFILE,settings.CERT_FILE)
        except AttributeError:
            pass
        l = ldap.initialize(settings.LDAP_URL)
        l.set_option(ldap.OPT_PROTOCOL_VERSION, 3)
        l.simple_bind_s(username, password)
        return l

    def authenticate(self,username=None,password=None):
        try:
            if len(password) == 0:
                return None
            dn = 'uid=%s,cn=users,%s' % (username,settings.SEARCH_DN)
            con = ldap.initialize(settings.LDAP_URL)
            con.set_option(ldap.OPT_X_TLS_DEMAND, True)
            con.set_option(ldap.OPT_PROTOCOL_VERSION,3)
            con.set_option(ldap.OPT_DEREF,3)
            con.simple_bind_s (dn,password)
            return self.get_or_create_user(dn, password)

        except ImportError:
            pass
        except ldap.INVALID_CREDENTIALS:
            pass

    def get_or_create_user(self, username, password):
        stripped_name = ''
        if username.lower().startswith('uid='):
            stripped_name = username.split(',')[0][4:].lower()
        try:
            user = User.objects.get(username=stripped_name)
        except User.DoesNotExist:
            try:
                l = self.bind_ldap(settings.BIND_USER, settings.BIND_PASSWORD)
                # search
                group_result = l.search_ext_s('%s,%s' % ('cn=groups',settings.SEARCH_DN),
                                        ldap.SCOPE_SUBTREE,
                                        "(cn=*)",
                                        ['cn','memberUid'])


                result = l.search_ext_s('%s,%s' % ('cn=users',settings.SEARCH_DN),
                                                    ldap.SCOPE_SUBTREE,
                                                    "(uid=%s)" % stripped_name,
                                                    settings.SEARCH_FIELDS)[0][1]
                l.unbind_s()

                membership = []
                for group in group_result:
                    if 'memberUid' in group[1] and stripped_name in group[1]['memberUid']:
                        membership.append("cn=%s,cn=groups,%s" % (group[1]['cn'][0], settings.SEARCH_DN))

                if len(membership) == 0:
                    membership = None

                # get email
                if 'mail' in result:
                    mail = result['mail'][0]
                else:
                    mail = ''
                # get surname
                if 'sn' in result:
                    last_name = result['sn'][0]
                else:
                    last_name = None

                # get display name
                if 'givenName' in result:
                    first_name = result['givenName'][0]
                else:
                    first_name = None
                user = User(username=stripped_name,first_name=first_name,last_name=last_name,email=mail)
            except Exception as e:
                return None

            user.is_staff = False
            user.is_superuser = False
            user.set_unusable_password()
            user.save()
            self.set_memberships_from_ldap(user, membership)

        return user




import ldap
import ldap.filter
from ldap_groups import settings

import simplejson as json

from django.http import HttpResponse, HttpResponseNotAllowed



class LDAPSearchResult(object):
    """
    Convenience bag class to help organize the results of an LDAP search.
    """
    def __init__(self, search_result):
        self.dn = search_result[0]
        self.cn = search_result[1]['cn'][0]

def search(canonical_name):
    """
    Searches the LDAP server for all Organizational Units (OUs) whose Canonical
    Name (CN) contains the text argument canonical_name passed into the
    function.
    """
    ldap.set_option(ldap.OPT_REFERRALS,0)
    l = ldap.initialize(settings.LDAP_URL)
    l.set_option(ldap.OPT_PROTOCOL_VERSION, 3)
    binddn = ''
    try:
        binddn = "%s@%s" % (settings.LDAP_BIND_USER, settings.LDAP_NT4_DOMAIN)
    except AttributeError:
        binddn = settings.LDAP_BIND_USER
    l.simple_bind_s(binddn, settings.LDAP_BIND_PASSWORD)
    base = settings.LDAP_SEARCH_DN
    scope = ldap.SCOPE_SUBTREE
    retrieve_attributes = ['cn']

    filtered_name = ldap.filter.escape_filter_chars(canonical_name)
    filter = 'cn=*%s*' % filtered_name

    results = l.search_s(base, scope, filter, retrieve_attributes)

    #result_objects = [LDAPSearchResult(result) for result in results]
    result_objects = []
    for result in results:
        if result[0]:
            result_objects.append(LDAPSearchResult(result))
    return result_objects

def ldap_search(request):
    """
    This view provides a JSON response of the Distinguished Names (DNs) returned
    by searching the LDAP server for OU name fragments.  One can search for e.g.
    'django' and retrieve all DNs with 'django' in the Canonical Name (CN).
    """
    if request.method != 'POST':
        return HttpResponseNotAllowed('POST')
    req_cn = None
    if 'req_cn' in request.POST:
        req_cn = request.POST['req_cn']
        results = search(req_cn)
        res_json = json.dumps([res.dn for res in results], ensure_ascii=False)

        return HttpResponse(res_json, mimetype='application/json')




from django.db import models
from django.contrib.auth.models import Group
from django.utils.encoding import python_2_unicode_compatible


@python_2_unicode_compatible
class LDAPGroup(models.Model):
    """
    An LDAPGroup defines a mapping between an LDAP Organizational Unit (OU) and
    a django.contrib.auth.models.Group group. The site admin will create these
    mappings in order to assign various levels of permissions to users who will
    be authenticating via LDAP rather than storing passwords in the Django
    database.  When a user authenticates via LDAP for the first time, the user's
    account in Django will be automatically assigned to the groups mapped by
    this model, and inherit the groups' permissions.
    """
    make_staff = models.BooleanField(default=False)
    make_superuser = models.BooleanField(default=False)
    org_unit = models.TextField()
    groups = models.ManyToManyField(Group, related_name='ldap_org_units')

    def __str__(self):
        return 'LDAP Groups for OU %s' % self.org_unit

    class Meta:
        ordering = ['org_unit']



from setuptools import setup, find_packages

setup(name='django-ldap-groups-bsc',
      version='0.1.8.3',
      author='Peter Herndon',
      author_email='tpherndon@gmail.com',
      url='http://code.google.com/p/django-ldap-groups/',
      description='A Django app for authenticating and authorizing against LDAP',
      packages=['ldap_groups', 'ldap_groups.accounts'],
      package_data={'ldap_groups': ['templates/admin/ldap_groups/ldapgroup/*']},
      classifiers=[
          'Development Status :: 4 - Beta',
          'Environment :: Web Environment',
          'Framework :: Django',
          'Intended Audience :: Developers',
          'Intended Audience :: System Administrators',
          'License :: OSI Approved :: BSD License',
          'Programming Language :: Python :: 2.6',
          'Programming Language :: Python :: 3.4',
          'Topic :: Internet :: WWW/HTTP',
          'Topic :: System :: Systems Administration :: Authentication/Directory :: LDAP',
          ]

      )



# -*- coding: utf-8 -*-
from __future__ import unicode_literals
from django.db import models, migrations


class Migration(migrations.Migration):

    dependencies = [
        ('auth', '0006_require_contenttypes_0002'),
    ]

    operations = [
        migrations.CreateModel(
            name='LDAPGroup',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('make_staff', models.BooleanField(default=False)),
                ('make_superuser', models.BooleanField(default=False)),
                ('org_unit', models.TextField()),
                ('groups', models.ManyToManyField(related_name='ldap_org_units', to='auth.Group')),
            ],
            options={
                'ordering': ['org_unit'],
            },
        ),
    ]



from django.conf import settings


# Ugly code is for legacy settings. We now prefer LDAP_ is always used.
LDAP_URL = getattr(settings, 'LDAP_URL', None)
LDAP_NT4_DOMAIN = getattr(settings, 'LDAP_NT4_DOMAIN', None)
if LDAP_NT4_DOMAIN == None:
    LDAP_NT4_DOMAIN = getattr(settings, 'NT4_DOMAIN', None)
LDAP_BIND_USER = getattr(settings, 'LDAP_BIND_USER', None)
if LDAP_BIND_USER == None:
    LDAP_BIND_USER = getattr(settings, 'BIND_USER', None)
LDAP_BIND_PASSWORD = getattr(settings, 'LDAP_BIND_PASSWORD', None)
if LDAP_BIND_PASSWORD == None:
    LDAP_BIND_PASSWORD = getattr(settings, 'BIND_PASSWORD', None)
LDAP_SEARCH_DN = getattr(settings, 'LDAP_SEARCH_DN', None)
if LDAP_SEARCH_DN == None:
    LDAP_SEARCH_DN = getattr(settings, 'SEARCH_DN', None)


# Use constance settings is available
# But fall back to settings.py
if getattr(settings, 'LDAP_USE_CONSTANCE', False) == True:
    from constance import config
    LDAP_URL = getattr(config, 'LDAP_URL', LDAP_URL)
    LDAP_NT4_DOMAIN = getattr(config, 'LDAP_NT4_DOMAIN', LDAP_NT4_DOMAIN)
    LDAP_BIND_USER = getattr(config, 'LDAP_BIND_USER', LDAP_BIND_USER)
    LDAP_BIND_PASSWORD = getattr(config, 'LDAP_BIND_PASSWORD', LDAP_BIND_PASSWORD)
    LDAP_SEARCH_DN = getattr(config, 'LDAP_SEARCH_DN', LDAP_SEARCH_DN)



from django.contrib import admin
from ldap_groups.models import LDAPGroup
from ldap_groups.views import ldap_search
from django.conf.urls import *

class LDAPGroupAdmin(admin.ModelAdmin):
    def get_urls(self):
        urls = super(LDAPGroupAdmin, self).get_urls()
        my_urls = patterns('',
            (r'^ldap_search/$', self.admin_site.admin_view(ldap_search)),
        )
        
        return my_urls + urls

admin.site.register(LDAPGroup, LDAPGroupAdmin)
